Package: GFDsurv
Type: Package
Title: Tests for Survival Data in General Factorial Designs
Version: 0.1.2
Authors@R: 
  c(person(given = "Marc",
           family = "Ditzhaus",
           role = c("aut")),
    person(given = "Dennis",
           family = "Dobler",
           role = "aut"),
    person(given = "Markus",
           family = "Pauly",
           role = "aut"),
    person(given = "Philipp",
           family = "Steinhauer",
           role = "aut"),
    person(given = "Merle",
           family = "Munko",
           role = c("aut","cre"), email = "merle.munko@ovgu.de"))
URL: https://github.com/MerleMunko/GFDsurv
Description: Implemented are three Wald-type statistic and respective
  permuted versions for null hypotheses formulated in terms of cumulative hazard rate functions, medians and the concordance measure, respectively, in the general framework of survival factorial designs with possibly heterogeneous survival and/or censoring distributions, for crossed designs with an arbitrary number of factors and nested designs with up to three factors.
	Ditzhaus, Dobler and Pauly (2020) <doi:10.1177/0962280220980784> 
	Ditzhaus, Genuneit, Janssen, Pauly (2023) <doi:10.1111/biom.13575>
	Dobler and Pauly (2019) <doi:10.1177/0962280219831316>.
License: GPL (>= 3)
Imports: gridExtra (>= 2.3), shiny (>= 1.5.0), shinyjs (>= 2.0.0),
        shinythemes (>= 1.1.2), survival (>= 3.2-7), survminer (>=
        0.4.8), tippy (>= 0.1.0), magic (>= 1.5-9), MASS (>= 7.3-53),
        plyr (>= 1.8.6), stats
Encoding: UTF-8
Suggests: condSURV
NeedsCompilation: no
Packaged: 2026-05-13 13:25:59 UTC; munko
Author: Marc Ditzhaus [aut],
  Dennis Dobler [aut],
  Markus Pauly [aut],
  Philipp Steinhauer [aut],
  Merle Munko [aut, cre]
Maintainer: Merle Munko <merle.munko@ovgu.de>
Repository: CRAN
Date/Publication: 2026-05-14 14:20:30 UTC
Config/roxygen2/version: 8.0.0
Built: R 4.5.2; ; 2026-05-14 19:11:04 UTC; unix
