# Generated by roxygen2: do not edit by hand

S3method(plot,casanova)
S3method(plot,copsanova)
S3method(plot,medsanova)
S3method(print,casanova)
S3method(print,copsanova)
S3method(print,medsanova)
S3method(summary,casanova)
S3method(summary,copsanova)
S3method(summary,medsanova)
export(GFDsurvGUI)
export(casanova)
export(copsanova)
export(medsanova)
import(plyr)
import(shiny)
import(stats)
import(utils)
importFrom(MASS,ginv)
importFrom(gridExtra,grid.arrange)
importFrom(magic,adiag)
importFrom(shinyjs,hidden)
importFrom(shinyjs,hide)
importFrom(shinyjs,show)
importFrom(shinyjs,useShinyjs)
importFrom(shinythemes,shinytheme)
importFrom(survival,survfit)
importFrom(survminer,ggsurvplot)
importFrom(tippy,tippy_this)
